# -*- coding: utf-8 -*-

"""
 @Time    : 2019/3/12
 @Author  : tudou (upczyxl@163.com)
 @File    : __init__.py.py
"""
import os
import sys

current_dir = os.path.abspath(os.path.dirname(__file__))
sys.path.append(current_dir)
sys.path.append("..")